@@ -1,33 +1,52 @@
+import enum
 import os
 import re
 import tokenize
+from argparse import Namespace
+from ast import AST
+from pathlib import Path
 from string import ascii_lowercase, ascii_uppercase, digits
+from tokenize import TokenInfo
+from typing import Any, FrozenSet, Iterable, Iterator, List, Optional, Tuple, Type
 
-import pkg_resources
+from flake8.options.manager import OptionManager
+
+from .version import version as __version__
 
 NOQA_REGEX = re.compile(r"#[\s]*noqa:[\s]*[\D]+[\d]+")
+DICTIONARY_PATH = Path(__file__).parent
+
+
+LintError = Tuple[int, int, str, Type["SpellCheckPlugin"]]
+Position = Tuple[int, int]
+
+
+class WordCase(enum.Enum):
+    URL = enum.auto()
+    SNAKE = enum.auto()
+    CAMEL = enum.auto()
 
 
 # Really simple detection function
-def detect_case(name):
-    if name.startswith("http"):
-        return "url"
+def detect_case(word: str) -> WordCase:
+    if word.startswith("http"):
+        return WordCase.URL
     # ignore leading underscores when testing for snake case
-    elif "_" in name.lstrip("_"):
-        return "snake"
-    elif name.isupper():
-        return "snake"
+    elif "_" in word.lstrip("_"):
+        return WordCase.SNAKE
+    elif word.isupper():
+        return WordCase.SNAKE
     else:
-        return "camel"
+        return WordCase.CAMEL
 
 
-def parse_camel_case(name, position):
+def parse_camel_case(name: str, position: Position) -> Iterator[Tuple[Position, str]]:
     index = position[1]
     start = index
     buffer = ""
     for c in name:
         index += 1
-        if c in ascii_lowercase or c in digits or c in ("'"):
+        if c in ascii_lowercase or c in digits or c in "'":
             buffer += c
         else:
             if buffer:
@@ -43,7 +62,7 @@ def parse_camel_case(name, position):
         yield (position[0], start), buffer
 
 
-def parse_snake_case(name, position):
+def parse_snake_case(name: str, position: Position) -> Iterator[Tuple[Position, str]]:
     index = position[1]
     start = index
     buffer = ""
@@ -62,7 +81,7 @@ def parse_snake_case(name, position):
         yield (position[0], start), buffer
 
 
-def is_number(value):
+def is_number(value: Any) -> bool:
     try:
         float(value)
     except ValueError:
@@ -71,45 +90,61 @@ def is_number(value):
         return True
 
 
-def get_code(token_type):
+def get_code(token_type: int) -> str:
     if token_type == tokenize.COMMENT:
         return "SC100"
     elif token_type == tokenize.NAME:
         return "SC200"
     else:
-        raise ValueError("Unknown token_type {}".format(token_type))
+        raise ValueError(f"Unknown token_type {token_type}")
 
 
 class SpellCheckPlugin:
     name = "flake8-spellcheck"
-    version = "0.22.0"
+    version = __version__
 
-    def __init__(self, tree, filename="(none)", file_tokens=None):
-        self.file_tokens = file_tokens
+    spellcheck_targets: FrozenSet[str] = frozenset()
+    no_symbols: FrozenSet[str] = frozenset()
+    words: FrozenSet[str] = frozenset()
 
-        self.words = set()
-        for dictionary in self.dictionaries:
-            data = pkg_resources.resource_string(__name__, dictionary)
-            data = data.decode("utf8")
-            self.words |= set(w.lower() for w in data.split("\n"))
-
-        if os.path.exists(self.whitelist_path):
-            with open(self.whitelist_path, "r") as fp:
-                whitelist = fp.read()
-
-            whitelist = set(w.lower() for w in whitelist.split("\n"))
-            self.words |= whitelist
-
-        # Hacky way of getting dictionary with symbols stripped
-        self.no_symbols = set()
-        for w in self.words:
-            if w.endswith("'s"):
-                self.no_symbols.add(w.replace("'s", ""))
-            else:
-                self.no_symbols.add(w.replace("'", ""))
+    def __init__(
+        self,
+        tree: AST,
+        filename: str = "(none)",
+        file_tokens: Optional[Iterable[TokenInfo]] = None,
+    ) -> None:
+        if file_tokens is None:
+            raise ValueError("Plugin requires file_tokens")
+        else:
+            self.file_tokens: Iterable[TokenInfo] = file_tokens
 
     @classmethod
-    def add_options(cls, parser):
+    def load_dictionaries(
+        cls, options: Namespace
+    ) -> Tuple[FrozenSet[str], FrozenSet[str]]:
+        words = set()
+        for dictionary_name in options.dictionaries:
+            dictionary_path = DICTIONARY_PATH / f"{dictionary_name}.txt"
+            data = dictionary_path.read_text()
+            words |= {w.lower() for w in data.split("\n")}
+
+        if os.path.exists(options.whitelist):
+            with open(options.whitelist) as fp:
+                whitelist = fp.read()
+            whitelist_data = {w.lower() for w in whitelist.split("\n")}
+            words |= whitelist_data
+
+        # Hacky way of getting dictionary with symbols stripped
+        no_symbols = set()
+        for w in words:
+            if w.endswith("'s"):
+                no_symbols.add(w.replace("'s", ""))
+            else:
+                no_symbols.add(w.replace("'", ""))
+        return frozenset(words), frozenset(no_symbols)
+
+    @classmethod
+    def add_options(cls, parser: OptionManager) -> None:
         parser.add_option(
             "--whitelist",
             help="Path to text file containing whitelisted words",
@@ -134,12 +169,13 @@ class SpellCheckPlugin:
         )
 
     @classmethod
-    def parse_options(cls, options):
-        cls.whitelist_path = options.whitelist
-        cls.dictionaries = [d + ".txt" for d in options.dictionaries]
-        cls.spellcheck_targets = set(options.spellcheck_targets)
+    def parse_options(cls, options: Namespace) -> None:
+        cls.words, cls.no_symbols = cls.load_dictionaries(options)
+        cls.spellcheck_targets = frozenset(options.spellcheck_targets)
 
-    def _detect_errors(self, tokens, use_symbols, token_type):
+    def _detect_errors(
+        self, tokens: Iterable[Tuple[Position, str]], use_symbols: bool, token_type: int
+    ) -> Iterator[LintError]:
         code = get_code(token_type)
 
         for position, token in tokens:
@@ -155,15 +191,15 @@ class SpellCheckPlugin:
                 yield (
                     position[0],
                     position[1],
-                    "{} Possibly misspelt word: '{}'".format(code, token),
+                    f"{code} Possibly misspelt word: '{token}'",
                     type(self),
                 )
 
-    def run(self):
+    def run(self) -> Iterator[LintError]:
         for token_info in self.file_tokens:
             yield from self._parse_token(token_info)
 
-    def _is_valid_comment(self, token_info):
+    def _is_valid_comment(self, token_info: tokenize.TokenInfo) -> bool:
         return (
             token_info.type == tokenize.COMMENT
             and "comments" in self.spellcheck_targets
@@ -174,7 +210,7 @@ class SpellCheckPlugin:
             and token_info.string.lstrip("#").split()[0] != "noqa:"
         )
 
-    def _parse_token(self, token_info):
+    def _parse_token(self, token_info: tokenize.TokenInfo) -> Iterator[LintError]:
         if token_info.type == tokenize.NAME and "names" in self.spellcheck_targets:
             value = token_info.string
         elif self._is_valid_comment(token_info):
@@ -184,21 +220,25 @@ class SpellCheckPlugin:
         else:
             return
 
-        tokens = []
+        tokens: List[Tuple[Position, str]] = []
         for word in value.split(" "):
             case = detect_case(word)
-            if case == "url":
+            if case == WordCase.URL:
                 # Nothing to do here
                 continue
-            elif case == "snake":
+            elif case == WordCase.SNAKE:
                 tokens.extend(parse_snake_case(word, token_info.start))
-            elif case == "camel":
+            elif case == WordCase.CAMEL:
                 tokens.extend(parse_camel_case(word, token_info.start))
 
         if token_info.type == tokenize.NAME:
             use_symbols = False
         elif token_info.type == tokenize.COMMENT:
             use_symbols = True
+        else:
+            return
 
-        for error_tuple in self._detect_errors(tokens, use_symbols, token_info.type):
-            yield error_tuple
+        yield from self._detect_errors(tokens, use_symbols, token_info.type)
+
+
+__all__ = ("__version__", "SpellCheckPlugin")
